import 'package:flutter/cupertino.dart';

abstract class IPagedProvider<ItemType> {
  final List<ItemType> content = <ItemType>[];
  int currentPage = 0;
  bool lastPageFetched = false;
  bool loadingPage = false;

  Future<void> scrollEndHandler(
      bool endOfList, void Function(VoidCallback fn) setState) async {
    if ((endOfList || content.isEmpty) && canLoadnewPages) {
      loadNewPage(setState);
    }
  }

  Future<void> loadNewPage(void Function(VoidCallback fn) setState) {
    loadingPage = true;
    return tryLoadingNewPage(setState).whenComplete(() {
      loadingPage = false;
    });
  }

  bool get canLoadnewPages => !lastPageFetched && !loadingPage;

  @protected
  Future<void> tryLoadingNewPage(void Function(VoidCallback fn) setState);

  Future<List<ItemType>> fetchPage(int page);
}
