import 'package:flutter/cupertino.dart';
import 'package:yapalivi/providers/IPagedProvider.dart';

class Yapalivi<ItemType> extends StatefulWidget {
  final IPagedProvider<ItemType> provider;
  final Widget Function(BuildContext context, int index, ItemType item)
      itemBuilder;
  final Widget? Function(
      BuildContext context,
      int previousIndex,
      int followingIndex,
      ItemType previousItem,
      ItemType followingItem)? separatorBuilder;
  final Widget? placeholderWidget;
  final Widget? noMoreItemsWidget;
  final Iterable<ItemType> Function(List<ItemType> elements)? filterFunction;

  const Yapalivi(
      {Key? key,
      required this.provider,
      required this.itemBuilder,
      this.separatorBuilder,
      this.placeholderWidget,
      this.noMoreItemsWidget,
      this.filterFunction})
      : super(key: key);

  @override
  _YapaliviState<ItemType> createState() => _YapaliviState<ItemType>();
}

class _YapaliviState<ItemType> extends State<Yapalivi<ItemType>> {
  Iterable<ItemType> get content {
    return widget.filterFunction?.call(widget.provider.content) ??
        widget.provider.content;
  }

  int get listLength {
    return content.length +
        (!widget.provider.lastPageFetched && widget.placeholderWidget != null
            ? 1
            : 0) +
        (widget.provider.lastPageFetched && widget.noMoreItemsWidget != null
            ? 1
            : 0);
  }

  Widget getitemBuilder(int index) {
    if (index >= content.length) {
      if (widget.provider.lastPageFetched && widget.noMoreItemsWidget != null)
        return widget.noMoreItemsWidget!;
      else if (widget.placeholderWidget != null)
        return widget.placeholderWidget!;
    }
    return widget.itemBuilder(context, index, content.elementAt(index));
  }

  @override
  void initState() {
    super.initState();
    widget.provider.loadNewPage(setState);
  }

  @override
  Widget build(BuildContext context) {
    return NotificationListener<ScrollEndNotification>(
      child: widget.separatorBuilder == null
          ? ListView.builder(
              itemBuilder: (context, index) => getitemBuilder(index),
              itemCount: listLength)
          : ListView.separated(
              itemBuilder: (context, index) => getitemBuilder(index),
              separatorBuilder: (context, index) =>
                  widget.separatorBuilder!.call(context, index, index + 1,
                      content.elementAt(index), content.elementAt(index)) ??
                  Container(),
              itemCount: listLength),
      onNotification: (notification) {
        if (notification.metrics.atEdge) if (notification.metrics.pixels == 0.0)
          widget.provider.scrollEndHandler(false, setState);
        else
          widget.provider.scrollEndHandler(true, setState);
        return true;
      },
    );
  }
}
